
(function(root) {

  if (typeof exports === 'object') {
    var Fraction = require('fraction.js');
    var Complex = require('complex.js');
  } else {
    var Fraction = root.Fraction;
    var Complex = root.Complex;
  }

  function egcd(a, b) {

    var s = 0, t = 1, u = 1, v = 0;
    while (a !== 0) {

      var q = b / a | 0, r = b % a;
      var m = s - u * q, n = t - v * q;

      b = a;
      a = r;
      s = u;
      t = v;
      u = m;
      v = n;
    }
    return [b, s, t];
  }

  function mod(n, m) {

    return (n % m + m) % m;
  }

  function modinv(z, n) {

    var tmp = egcd(z, n);
    if (tmp[0] !== 1) {
      return null;
    }
    return mod(tmp[1], n);
  }

  function Matrix(rows, cols, field) {

    this.field = field;
    this.matrix = new Array(rows);
    for (var i = 0; i < rows; i++) {
      this.matrix[i] = new Array(cols);
    }
  }

  Matrix.FIELDS = {
    C: {
      parse: function(v) {
        return new Complex(v);
      },
      add: function(a, b) {
        return a.add(b);
      },
      mul: function(a, b) {
        return a.mul(b);
      },
      zero: function() {
        return new Complex(0);
      },
      equals: function(a, b) {
        return a.equals(b);
      },
      inverse: function(a) {
        return a.inverse();
      },
      negate: function(a) {
        return a.neg();
      }
    },
    R: {
      parse: function(v) {
        v = parseFloat(v);
        if (isNaN(v))
          throw new Error("Invalid number");
        return v;
      },
      add: function(a, b) {
        return a + b;
      },
      mul: function(a, b) {
        return a * b;
      },
      zero: function() {
        return 0;
      },
      equals: function(a, b) {
        return Math.abs(a - b) < 1e-15;
      },
      inverse: function(a) {
        return 1 / a;
      },
      negate: function(a) {
        return -a;
      }
    },
    Q: {
      parse: function(v) {
        return new Fraction(v);
      },
      add: function(a, b) {
        return a.add(b);
      },
      mul: function(a, b) {
        return a.mul(b);
      },
      zero: function() {
        return new Fraction(0);
      },
      equals: function(a, b) {
        return a.equals(b);
      },
      inverse: function(a) {
        return a.inverse();
      },
      negate: function(a) {
        return a.neg();
      }
    },
    Z: function(prime) {

      return {
        parse: function(v) {
          var v = parseInt(v, 10);
          if (isNaN(v))
            throw new Error("Invalid number");
          return v;
        },
        add: function(a, b) {
          return mod(a + b, prime);
        },
        mul: function(a, b) {
          return mod(a * b, prime);
        },
        zero: function() {
          return 0;
        },
        equals: function(a, b) {
          return mod(a, prime) === mod(b, prime);
        },
        inverse: function(a) {

          var inv = modinv(a, prime);
          if (inv === null) {
            throw "Modular inverse does not exist.";
          }
          return inv;
        },
        negate: function(a) {
          return mod(-a, prime);
        }
      };
    }
  };

  Matrix.prototype = {
    matrix: null,
    field: null,
    rowCount: function() {
      return this.matrix.length;
    },
    colCount: function() {
      return this.matrix[0].length;
    },
    get: function(row, col) {
      return this.matrix[row][col];
    },
    set: function(row, col, val) {
      this.matrix[row][col] = this.field.parse(val);
    },
    transpose: function() {
      var rows = this.rowCount();
      var cols = this.colCount();
      var res = new Matrix(cols, rows, this.field);
      for (var i = 0; i < rows; i++) {
        for (var j = 0; j < cols; j++) {
          res[j][i] = this.matrix[i][j];
        }
      }
    },
    swapRows: function(r0, r1) {
      var tmp = this.matrix[r0];
      this.matrix[r0] = this.matrix[r1];
      this.matrix[r1] = tmp;
    },
    mulRow: function(row, factor) {
      var cols = this.colCount();
      for (var i = 0; i < cols; i++) {
        this.set(row, i, this.field.mul(this.get(row, i), factor));
      }
    },
    addRows: function(srcRow, dstRow, factor) { // dst+= src * factor
      var cols = this.colCount();
      for (var i = 0; i < cols; i++) {
        this.set(dstRow, i, this.field.add(this.get(dstRow, i), this.field.mul(this.get(srcRow, i), factor)));
      }
    },
    mul: function(other) {
      if (this.colCount() !== other.rowCount()) {
        throw new Error("Invalid dimensions");
      }
      var rows = this.rowCount();
      var cols = other.colCount();
      var cells = this.colCount();

      var res = new Matrix(rows, cols, this.field);
      for (var i = 0; i < rows; i++) {
        for (var j = 0; j < cols; j++) {
          var sum = this.field.zero();
          for (var k = 0; k < cells; k++) {
            sum = this.field.add(this.field.mul(this.get(i, k), other.get(k, j)), sum);
          }
          res.set(i, j, sum);
        }
      }
      return res;
    },
    toString: function() {
      var rows = this.rowCount();
      var cols = this.colCount();
      var str = "";
      for (var i = 0; i < rows; i++) {

        for (var j = 0; j < cols; j++) {
          str += this.matrix[i][j].toString();
          str += "\t";
        }
        str += "\n";
      }
      return str;
    },
    reducedRowEchelonForm: function() {
      var rows = this.rowCount();
      var cols = this.colCount();

      // Compute row echelon form (REF)
      var numPivots = 0;
      for (var j = 0; j < cols && numPivots < rows; j++) {  // For each column
        // Find a pivot row for this column
        var pivotRow = numPivots;
        while (pivotRow < rows && this.field.equals(this.get(pivotRow, j), this.field.zero()))
          pivotRow++;
        if (pivotRow === rows)
          continue;  // Cannot eliminate on this column
        this.swapRows(numPivots, pivotRow);
        pivotRow = numPivots;
        numPivots++;

        // Simplify the pivot row
        this.mulRow(pivotRow, this.field.inverse(this.get(pivotRow, j)));

        // Eliminate rows below
        for (var i = pivotRow + 1; i < rows; i++) {
          this.addRows(pivotRow, i, this.field.negate(this.get(i, j)));
        }
      }

      // Compute reduced row echelon form (RREF)
      for (var i = numPivots - 1; i >= 0; i--) {
        // Find pivot
        var pivotCol = 0;
        while (pivotCol < cols && this.field.equals(this.get(i, pivotCol), this.field.zero()))
          pivotCol++;
        if (pivotCol === cols)
          continue;  // Skip this all-zero row

        // Eliminate rows above
        for (var j = i - 1; j >= 0; j--) {
          this.addRows(i, j, this.field.negate(this.get(j, pivotCol)));
        }
      }
    }
  };

  if (typeof define === 'function' && define['amd']) {
    define([], function() {
      return Matrix;
    });
  } else if (typeof exports === 'object') {
    module['exports'] = Matrix;
  } else {
    root['Matrix'] = Matrix;
  }
})(this);













/**
* Replaces the values of this matrix with the inverse of this matrix. Requires the matrix to be square.
* All elements of this matrix should be non-{@code null} when performing this operation.
* Throws an exception if the matrix is singular (not invertible). If an exception is thrown, this matrix is unchanged.
* The time complexity of this operation is <var>O</var>(rows<sup>3</sup>).
* @throws IllegalStateException if this matrix is not square
* @throws IllegalStateException if this matrix has no inverse
*//*
public void invert() {
int rows = rowCount();
int cols = columnCount();
if (rows != cols)
throw new IllegalStateException("Matrix dimensions are not square");

// Build augmented matrix [this | identity]
Matrix<E> temp = new Matrix<>(rows, cols * 2, f);
    for (int i = 0; i < rows; i++) {
    for (int j = 0; j < cols; j++) {
    temp.set(i, j, get(i, j));
    temp.set(i, j + cols, i == j ? f.one() : f.zero());
    }
    }

    // Do the main calculation
    temp.reducedRowEchelonForm();

    // Check that the left half is the identity matrix
    for (int i = 0; i < rows; i++) {
    for (int j = 0; j < cols; j++) {
    if (!f.equals(temp.get(i, j), i == j ? f.one() : f.zero()))
    throw new IllegalStateException("Matrix is not invertible");
    }
    }

    // Extract inverse matrix from: [identity | inverse]
    for (int i = 0; i < rows; i++) {
    for (int j = 0; j < cols; j++)
    set(i, j, temp.get(i, j + cols));
    }
    }


    /**
    * Returns the determinant of this matrix, and as a side effect converts the matrix to row echelon form (REF).
    * Requires the matrix to be square. The leading coefficient of each row is not guaranteed to be one.
    * All elements of this matrix should be non-{@code null} when performing this operation.
    * Always succeeds, as long as the field follows the mathematical rules and does not throw an exception.
    * The time complexity of this operation is <var>O</var>(rows<sup>3</sup>).
    * @return the determinant of this matrix
    * @throws IllegalStateException if this matrix is not square
    *//*
    public E determinantAndRef() {
    int rows = rowCount();
    int cols = columnCount();
    if (rows != cols)
    throw new IllegalStateException("Matrix dimensions are not square");
    E det = f.one();

    // Compute row echelon form (REF)
    int numPivots = 0;
    for (int j = 0; j < cols; j++) {  // For each column
    // Find a pivot row for this column
    int pivotRow = numPivots;
    while (pivotRow < rows && f.equals(get(pivotRow, j), f.zero()))
    pivotRow++;

    if (pivotRow < rows) {
    // This column has a nonzero pivot
    if (numPivots != pivotRow) {
    swapRows(numPivots, pivotRow);
    det = f.negate(det);
    }
    pivotRow = numPivots;
    numPivots++;

    // Simplify the pivot row
    E temp = get(pivotRow, j);
    multiplyRow(pivotRow, f.reciprocal(temp));
    det = f.multiply(temp, det);

    // Eliminate rows below
    for (int i = pivotRow + 1; i < rows; i++)
    addRows(pivotRow, i, f.negate(get(i, j)));
    }

    // Update determinant
    det = f.multiply(get(j, j), det);
    }
    return det;
    }

    }
*/
